package com.millicom.microservice.service.adapters.controlador.numero.temporal;


import com.millicom.microservice.service.adapters.dto.common.OperationException;
import com.millicom.microservice.service.external.services.personalizado.ConsultarResponse;
import com.millicom.microservice.service.service.numero.temporal.Consultar;
import com.readinessit.library.logging.application.services.LoggerService;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.xml.bind.*;
import jakarta.xml.soap.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RestController;
import org.w3c.dom.Document;
import org.w3c.dom.ls.DOMImplementationLS;
import org.w3c.dom.ls.LSSerializer;
import org.xml.sax.SAXException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.stream.StreamSource;
import java.io.*;
import java.nio.charset.StandardCharsets;
import java.util.Iterator;

import static com.millicom.microservice.service.external.services.common.Constants.MODULO;


@RestController
public class NumeroTemporalRecurso implements NumeroTemporalAdaptador {


    private final Consultar consultar;

    @Value("classpath:wsdl/MAIN.wsdl")
    private Resource recurso;

    @Autowired
    public NumeroTemporalRecurso(Consultar consultar) {
        this.consultar = consultar;
    }

    /**
     * Obtiene información de una Simcard.
     *
     * @param cuerpoSolicitud Datos de la solicitud.
     * @return ResponseEntity con la respuesta.
     */
    @Override
    public ResponseEntity<Object> consultarNumeroTemporal(String cuerpoSolicitud, HttpServletRequest solicitudHttp) {
        try {
            SOAPMessage mensajeSoap = obtenerSoapMessageDeXml(cuerpoSolicitud);
            SOAPElement elementoSoap = obtenerPrimerSoapElement(mensajeSoap.getSOAPBody());
            if (elementoSoap != null) {
                if (elementoSoap.getLocalName().equals("Consultar")) {
                    String xml = obtenerXmlDeSoapElement(elementoSoap, false, true);
                    com.millicom.microservice.service.external.services.Consultar solicitud = obtenerObjetoDeXml(xml, com.millicom.microservice.service.external.services.Consultar.class, false);
                    com.millicom.microservice.service.external.services.personalizado.ConsultarResponse response = consultar.consultar(solicitud.getConsultaRequest());
                    return new ResponseEntity<>(obtenerSoapMessage(response), null, HttpStatus.OK);
                } else {
                    return new ResponseEntity<>(obtenerSoapFault("Metodo no implementado"), HttpStatus.OK);
                }
            }

        } catch (OperationException e) {
            try {
                return new ResponseEntity<>(obtenerSoapFault(e.getErrorStructure().getValor()), e.getErrorStatus());
            } catch (Exception ex) {
                LoggerService.logBusiness(MODULO, "Exception: " + e.getMessage(), e);
            }
        } catch (Exception e) {
            try {
                return new ResponseEntity<>(obtenerSoapFault(e.toString()), HttpStatus.BAD_GATEWAY);
            } catch (Exception ex) {
                LoggerService.logBusiness(MODULO, "Exception: " + e.getMessage(), e);
            }
        }
        return ResponseEntity.badRequest().body("");
    }


    @Override
    public ResponseEntity<Object> obtenerFirma(HttpServletRequest request) {
        try {
            // Obtener el esquema (http o https)
            String scheme = request.getHeader("X-Forwarded-Proto");
            if (scheme == null) {
                scheme = request.getScheme(); // Usa el esquema de la solicitud si no está presente en los encabezados
            }

            // Obtener el nombre del servidor (dominio)
            String serverName = request.getHeader("X-Forwarded-Host");
            if (serverName == null) {
                serverName = request.getServerName(); // Usa el nombre del servidor de la solicitud si no está presente en los encabezados
            }

            // Obtener el puerto (si no es el puerto por defecto)
            String port = request.getHeader("X-Forwarded-Port");
            if (port == null) {
                int serverPort = request.getServerPort();
                // El puerto 80 para HTTP y 443 para HTTPS son por defecto y generalmente no se muestran
                if (serverPort == 80 || serverPort == 443) {
                    port = "";
                } else {
                    port = ":" + serverPort;
                }
            } else {
                port = ":" + port;
            }

            // Obtener el contexto de la aplicación
            String contextPath = request.getContextPath();
            // Obtener la URI completa
            String requestUri = request.getRequestURI();
            // Obtener los parámetros de la consulta (query string)
            String queryString = request.getQueryString();

            // Construir la URL absoluta
            String absoluteUrl = scheme + "://" + serverName + port + contextPath + requestUri;
            if (queryString != null) {
                absoluteUrl += "?" + queryString;
            }
            String firma = new String(recurso.getInputStream().readAllBytes(), StandardCharsets.UTF_8);
            firma = firma.replace("LOCALHOST", absoluteUrl);
            return new ResponseEntity<>(firma, HttpStatus.OK);
        } catch (Exception e) {
            return new ResponseEntity<>(e.getMessage(), HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }


    /**
     * Permite convertir a un Objeto serializable a paratir de un xml
     *
     * @param xml           Objeto a convertir en formato xml
     * @param clase         Clase al cual se va convertir el xml
     * @param conJaxElement Indica si va convertir a objeto usando JAXElement
     * @param <T>           Objeto generico a convertir el xml
     * @return Retorna un Objeto de tipo clase
     * @throws JAXBException
     */
    public static <T> T obtenerObjetoDeXml(String xml, Class clase, Boolean conJaxElement) throws JAXBException {
        StringReader sr = new StringReader(xml);
        JAXBContext jaxbContext = JAXBContext.newInstance(clase);
        Unmarshaller unmarshaller = jaxbContext.createUnmarshaller();
        if (Boolean.TRUE.equals(conJaxElement)) {
            return (T) ((JAXBElement) unmarshaller.unmarshal(sr)).getValue();
        } else {
            return (T) unmarshaller.unmarshal(sr);
        }
    }

    /**
     * Permite convertir un xml a partir de un SoapElement
     *
     * @param elemento       Objeto SOAPElement
     * @param declaracionXml Indicador que señala si el xml tendrá el tag de declaracion xml
     * @param tabularXml     Indicador que señala si el xml estará formateado
     * @return Retorna un xml de un SoapElement
     */
    private String obtenerXmlDeSoapElement(SOAPElement elemento, boolean declaracionXml, boolean tabularXml) {
        DOMImplementationLS implementacionDom = (DOMImplementationLS) elemento.getOwnerDocument().getImplementation().getFeature("LS", "3.0");
        LSSerializer serializador = implementacionDom.createLSSerializer();
        serializador.getDomConfig().setParameter("xml-declaration", declaracionXml);
        serializador.getDomConfig().setParameter("format-pretty-print", tabularXml);
        return serializador.writeToString(elemento);
    }


    /**
     * Permite obtener en formato texto un SoapFault a partir del detalle del error
     *
     * @param detalle Detalle del error con el cual se generará el SoapFault
     * @return Retorna un SoapFault en formato texto
     * @throws SOAPException
     * @throws IOException
     */
    private String obtenerSoapFault(String detalle) throws SOAPException, IOException {
        SOAPMessage mensajeSoap = MessageFactory.newInstance().createMessage();
        mensajeSoap.getSOAPPart().getEnvelope().getBody().addFault().setFaultString(detalle);
        ByteArrayOutputStream flujoSalida = new ByteArrayOutputStream();
        mensajeSoap.writeTo(flujoSalida);
        return flujoSalida.toString(StandardCharsets.UTF_8);
    }

    /**
     * Permite obtener un objeto SoapMessage a partir de un xml
     *
     * @param requestXml SoapMessage en formato texto
     * @return retorna un objeto SoapMessage a partir de un xml
     * @throws SOAPException
     */
    private SOAPMessage obtenerSoapMessageDeXml(String requestXml) throws SOAPException {
        SOAPMessage mensajeSoap = MessageFactory.newInstance().createMessage();
        SOAPPart seccionSoap = mensajeSoap.getSOAPPart();
        InputStream flujoEntrada = new ByteArrayInputStream(requestXml.getBytes(StandardCharsets.UTF_8));
        StreamSource flujoOrigen = new StreamSource(flujoEntrada);
        seccionSoap.setContent(flujoOrigen);
        mensajeSoap.saveChanges();
        return mensajeSoap;
    }

    /**
     * Permite obtener el primer objeto SoapElement del cuerpo de un SoapMessage
     *
     * @param cuerpoSoap Objeto SoapBody
     * @return Retorna el primer objeto de tipo SoapElement de un SoapBody
     */
    private SOAPElement obtenerPrimerSoapElement(SOAPBody cuerpoSoap) {
        for (Iterator<?> iterador = cuerpoSoap.getChildElements(); iterador.hasNext(); ) {
            Object hijo = iterador.next();
            if (hijo instanceof SOAPElement) {
                return (SOAPElement) hijo;
            }
        }
        return null;
    }

    private String convertirAXml(ConsultarResponse response) throws JAXBException {
        JAXBContext context = JAXBContext.newInstance(ConsultarResponse.class);
        Marshaller marshaller = context.createMarshaller();
        marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
        marshaller.setProperty(Marshaller.JAXB_FRAGMENT, Boolean.FALSE);
        StringWriter writer = new StringWriter();
        marshaller.marshal(response, writer);
        return writer.toString();
    }

    private Document obtenerDocumento(String xmlString) throws ParserConfigurationException, IOException, SAXException {
        DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
        dbFactory.setNamespaceAware(true);
        DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
        return dBuilder.parse(new ByteArrayInputStream(xmlString.getBytes(StandardCharsets.UTF_8)));
    }

    private String obtenerSoapMessage(ConsultarResponse response) throws SOAPException, IOException, JAXBException, ParserConfigurationException, SAXException {
        SOAPMessage mensajeSoap = MessageFactory.newInstance().createMessage();
        String xml = convertirAXml(response);
        LoggerService.logBusiness(MODULO, xml, null);
        Document documento = obtenerDocumento(xml);
        mensajeSoap.getSOAPPart().getEnvelope().getBody().addDocument(documento);
        mensajeSoap.saveChanges();
        ByteArrayOutputStream flujoSalida = new ByteArrayOutputStream();
        mensajeSoap.writeTo(flujoSalida);
        return new String(flujoSalida.toByteArray(), StandardCharsets.UTF_8);
    }
}
