package com.millicom.microservice.service.service.messaging;


import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.millicom.microservice.service.adapters.dto.common.ErrorStructure;
import com.millicom.microservice.service.service.common.GenerateErrorService;
import com.readinessit.library.logging.application.services.LoggerService;
import com.readinessit.library.messaging.data.MessageResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.HttpStatusCode;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import java.util.List;

import static com.millicom.microservice.service.external.services.common.Constants.*;

@Service
public class ErrorMapper {

    private final GenerateErrorService generateErrorService;
    private final ObjectMapper mapper;

    @Value("${service.numerotemporal.codigos.http.a.controlar}")
    private List<Integer> httpCodes;

    @Autowired
    public ErrorMapper(GenerateErrorService generateErrorService, ObjectMapper mapper) {
        this.generateErrorService = generateErrorService;
        this.mapper = mapper;
    }


    public ErrorStructure validate(Object response) {
        if (response == null) {
            return generateErrorService.generateError(HttpStatus.BAD_GATEWAY, ERROR_CODE_EXCEPTION,
                    new String[]{ERROR_DESC_EXCEPTION}, ERROR_DESC_EXCEPTION, this.getClass());
        } else if (response instanceof MessageResponse resp && resp.isError()) {
            if (httpCodes.contains(resp.getResponseStatus().value())) {

                ResponseEntity<Object> error = handleErrorResponseCreation(resp.getResponseHeaders(),
                        resp.getResponseStatus(), resp.getResponse());

                ErrorStructure errorStructure = error.hasBody() ? (ErrorStructure) error.getBody() : null;

                if (errorStructure != null && errorStructure.getValor() != null) {
                    return generateErrorService.generateError(resp.getResponseStatus(), errorStructure.toString(),
                            new String[]{errorStructure.toString()}, obtenerJsonDeMessageResponse(resp), getClass());
                }
                return errorStructure;
            } else {
                LoggerService.logBusiness(MODULO, "No se controlara el http: " + resp.getResponseStatus().toString(), null);
            }
        }
        return null;
    }


    private ResponseEntity<Object> handleErrorResponseCreation(HttpHeaders headers, HttpStatusCode status, Object response) {
        ErrorStructure error;
        try {
            error = mapper.convertValue(response, ErrorStructure.class);
            if (error == null || error.getValor() == null) {
                if (HttpStatus.INTERNAL_SERVER_ERROR.equals(status))
                    response = ERROR_DESC_EXCEPTION;
                String description;
                try {
                    description = response != null ? mapper.writeValueAsString(response) : null;
                } catch (JsonProcessingException e) {
                    LoggerService.logException(MODULO, "JsonProcessingException: " + e.getMessage(), e);
                    description = ERROR_DESC_EXCEPTION;
                }
                error = generateErrorService.generateError(status, ERROR_CODE_EXCEPTION, new String[]{ERROR_DESC_EXCEPTION}, description, getClass());
            }
        } catch (Exception ex) {
            error = generateErrorService.generateError(HttpStatus.INTERNAL_SERVER_ERROR, ERROR_CODE_EXCEPTION, new String[]{ERROR_DESC_EXCEPTION}, ERROR_DESC_EXCEPTION, getClass());
        }
        return new ResponseEntity<>(error, headers, status);
    }


    public String obtenerJsonDeMessageResponse(MessageResponse messageResponse) {
        try {
            return mapper.writeValueAsString(messageResponse.getResponse());
        } catch (JsonProcessingException jsonProcessingException) {
            LoggerService.logException(MODULO, "JsonProcessingException: " + jsonProcessingException.getMessage(), jsonProcessingException);
            // Manejar la excepción o registrarla si es necesario
            return null;
        }
    }


}

	