package com.millicom.microservice.service.service.numero.temporal;


import com.fasterxml.jackson.databind.ObjectMapper;
import com.millicom.microservice.service.adapters.Adapter;
import com.millicom.microservice.service.adapters.dto.common.OperationException;
import com.millicom.microservice.service.external.services.ConsultaRequest;
import com.millicom.microservice.service.external.services.personalizado.ConsultarResponse;
import com.millicom.microservice.service.service.messaging.ErrorMapper;
import com.millicom.microservice.service.service.numero.propiedades.Propiedades;
import com.readinessit.library.logging.application.services.LoggerService;
import com.readinessit.library.messaging.data.MessageResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.HashMap;

import static com.millicom.microservice.service.external.services.common.Constants.MODULO;


@Service
public class ConsultarImplementacion implements Consultar {

    private final Propiedades propiedades;
    private final Adapter adapter;
    private final ErrorMapper errorMapperImp;


    @Value("${service.numerotemporal.operacion}")
    private String operacion;
    @Value("${service.numerotemporal.sistema}")
    private String sistema;

    @Autowired
    public ConsultarImplementacion(Propiedades propiedades, Adapter adapter, ErrorMapper errorMapperImp) {
        this.propiedades = propiedades;
        this.adapter = adapter;
        this.errorMapperImp = errorMapperImp;
    }


    /**
     * Ejecuta una transacción.
     *
     * @param requestSoap La solicitud de transacción.
     * @return La respuesta de la transacción ejecutada.
     * @throws OperationException Si ocurre un error durante la ejecución de la transacción.
     */
    @Override
    public ConsultarResponse consultar(ConsultaRequest requestSoap) throws OperationException {
        long tiempoInicio = System.currentTimeMillis();
        MessageResponse respuestaServicio = enviarMensaje(requestSoap);
        var errorStructure = errorMapperImp.validate(respuestaServicio);
        if (errorStructure != null) {
            throw new OperationException(errorStructure.getHttpStatus(), errorStructure);
        }
        LoggerService.logBusiness(MODULO, "Tiempo obtener consultar: " + (System.currentTimeMillis() - tiempoInicio) + " ms.", null);
        if (respuestaServicio.getResponse() instanceof HashMap<?, ?>) {
            ObjectMapper objectMapper = new ObjectMapper();
            return objectMapper.convertValue(respuestaServicio.getResponse(), ConsultarResponse.class);
        }
        return (ConsultarResponse) respuestaServicio.getResponse();
    }


    /**
     * Envía un mensaje.
     *
     * @param pedido El pedido a enviar.
     * @return La respuesta del mensaje enviado.
     */
    private MessageResponse enviarMensaje(Object pedido) {
        return adapter.sendMessage(this.getClass(), operacion, pedido, propiedades.getMapPathVars(), propiedades.getMapHeader(), sistema);
    }


}
